@@ -23,18 +23,27 @@ class IRequest(Interface):
     @since: 9.0
     """
 
-    method = Attribute("A C{str} giving the HTTP method that was used.")
+    method = Attribute("A L{bytes} giving the HTTP method that was used.")
     uri = Attribute(
-        "A C{str} giving the full encoded URI which was requested (including "
-        "query arguments).")
+        "A L{bytes} giving the full encoded URI which was requested (including"
+        " query arguments).")
     path = Attribute(
-        "A C{str} giving the encoded query path of the request URI.")
+        "A L{bytes} giving the encoded query path of the request URI (not "
+        "including query arguments).")
     args = Attribute(
-        "A mapping of decoded query argument names as C{str} to "
-        "corresponding query argument values as C{list}s of C{str}.  "
-        "For example, for a URI with C{'foo=bar&foo=baz&quux=spam'} "
-        "for its query part, C{args} will be C{{'foo': ['bar', 'baz'], "
-        "'quux': ['spam']}}.")
+        "A mapping of decoded query argument names as L{bytes} to "
+        "corresponding query argument values as L{list}s of L{bytes}.  "
+        "For example, for a URI with C{foo=bar&foo=baz&quux=spam} "
+        "for its query part, C{args} will be C{{b'foo': [b'bar', b'baz'], "
+        "b'quux': [b'spam']}}.")
+
+    prepath = Attribute(
+        "The URL path segments which have been processed during resource "
+        "traversal, as a list of {bytes}.")
+
+    postpath = Attribute(
+        "The URL path segments which have not (yet) been processed "
+        "during resource traversal, as a list of L{bytes}.")
 
     requestHeaders = Attribute(
         "A L{http_headers.Headers} instance giving all received HTTP request "
@@ -42,8 +51,8 @@ class IRequest(Interface):
 
     content = Attribute(
         "A file-like object giving the request body.  This may be a file on "
-        "disk, a C{StringIO}, or some other type.  The implementation is free "
-        "to decide on a per-request basis.")
+        "disk, an L{io.BytesIO}, or some other type.  The implementation is "
+        "free to decide on a per-request basis.")
 
     responseHeaders = Attribute(
         "A L{http_headers.Headers} instance holding all HTTP response "
@@ -53,18 +62,26 @@ class IRequest(Interface):
         """
         Get an HTTP request header.
 
-        @type key: C{str}
+        @type key: L{bytes} or L{str}
         @param key: The name of the header to get the value of.
 
-        @rtype: C{str} or L{None}
+        @rtype: L{bytes} or L{str} or L{None}
         @return: The value of the specified header, or L{None} if that header
-            was not present in the request.
+            was not present in the request. The string type of the result
+            matches the type of C{key}.
         """
 
 
     def getCookie(key):
         """
         Get a cookie that was sent from the network.
+
+        @type key: L{bytes}
+        @param key: The name of the cookie to get.
+
+        @rtype: L{bytes} or L{None}
+        @returns: The value of the specified cookie, or L{None} if that cookie
+            was not present in the request.
         """
 
 
@@ -86,7 +103,7 @@ class IRequest(Interface):
         host we are listening on if the header is unavailable.
 
         @returns: the requested hostname
-        @rtype: C{str}
+        @rtype: L{str}
         """
 
 
@@ -105,6 +122,8 @@ class IRequest(Interface):
         The address may not be a network address.  Callers must check
         its type before using it.
 
+        @since: 18.4
+
         @return: the client's address.
         @rtype: an L{IAddress} provider.
         """
@@ -129,7 +148,7 @@ class IRequest(Interface):
         If no user was supplied, return the empty string.
 
         @returns: the HTTP user, if any
-        @rtype: C{str}
+        @rtype: L{str}
         """
 
 
@@ -140,7 +159,7 @@ class IRequest(Interface):
         If no password was supplied, return the empty string.
 
         @returns: the HTTP password, if any
-        @rtype: C{str}
+        @rtype: L{str}
         """
 
 
@@ -172,16 +191,21 @@ class IRequest(Interface):
 
     def URLPath():
         """
-        @return: A L{URLPath} instance which identifies the URL for which this
-            request is.
+        @return: A L{URLPath<twisted.python.urlpath.URLPath>} instance
+            which identifies the URL for which this request is.
         """
 
 
     def prePathURL():
         """
-        @return: At any time during resource traversal, a L{str} giving an
-            absolute URL to the most nested resource which has yet been
-            reached.
+        At any time during resource traversal or resource rendering,
+        returns an absolute URL to the most nested resource which has
+        yet been reached.
+
+        @see: {twisted.web.server.Request.prepath}
+
+        @return: An absolute URL.
+        @type: L{bytes}
         """
 
 
@@ -195,6 +219,9 @@ class IRequest(Interface):
     def getRootURL():
         """
         Get a previously-remembered URL.
+
+        @return: An absolute URL.
+        @type: L{bytes}
         """
 
 
@@ -210,6 +237,9 @@ class IRequest(Interface):
         Write some data to the body of the response to this request.  Response
         headers are written the first time this method is called, after which
         new response headers may not be added.
+
+        @param data: Bytes of the response body.
+        @type data: L{bytes}
         """
 
 
@@ -226,6 +256,9 @@ class IRequest(Interface):
     def setResponseCode(code, message=None):
         """
         Set the HTTP response code.
+
+        @type code: L{int}
+        @type message: L{bytes}
         """
 
 
@@ -234,11 +267,13 @@ class IRequest(Interface):
         Set an HTTP response header.  Overrides any previously set values for
         this header.
 
-        @type name: C{str}
-        @param name: The name of the header for which to set the value.
+        @type k: L{bytes} or L{str}
+        @param k: The name of the header for which to set the value.
 
-        @type value: C{str}
-        @param value: The value to set for the named header.
+        @type v: L{bytes} or L{str}
+        @param v: The value to set for the named header. A L{str} will be
+            UTF-8 encoded, which may not interoperable with other
+            implementations. Avoid passing non-ASCII characters if possible.
         """
 
 
@@ -285,7 +320,7 @@ class IRequest(Interface):
         tag given.
 
         @param etag: The entity tag for the resource being returned.
-        @type etag: C{str}
+        @type etag: L{str}
 
         @return: If I am a C{If-None-Match} conditional request and the tag
             matches one in the request, I return L{CACHED<http.CACHED>} to
@@ -363,7 +398,7 @@ class ICredentialFactory(Interface):
     an authorize requests.
     """
     scheme = Attribute(
-        "A C{str} giving the name of the authentication scheme with which "
+        "A L{str} giving the name of the authentication scheme with which "
         "this factory is associated.  For example, C{'basic'} or C{'digest'}.")
 
 
@@ -375,8 +410,8 @@ class ICredentialFactory(Interface):
         @param peer: The request the response to which this challenge will be
             included.
 
-        @rtype: C{dict}
-        @return: A mapping from C{str} challenge fields to associated C{str}
+        @rtype: L{dict}
+        @return: A mapping from L{str} challenge fields to associated L{str}
             values.
         """
 
@@ -385,7 +420,7 @@ class ICredentialFactory(Interface):
         """
         Create a credentials object from the given response.
 
-        @type response: C{str}
+        @type response: L{str}
         @param response: scheme specific response string
 
         @type request: L{twisted.web.http.Request}
@@ -431,7 +466,7 @@ class IBodyProducer(IPushProducer):
 
     length = Attribute(
         """
-        C{length} is a C{int} indicating how many bytes in total this
+        C{length} is a L{int} indicating how many bytes in total this
         L{IBodyProducer} will write to the consumer or L{UNKNOWN_LENGTH}
         if this is not known in advance.
         """)
@@ -441,8 +476,9 @@ class IBodyProducer(IPushProducer):
         Start producing to the given
         L{IConsumer<twisted.internet.interfaces.IConsumer>} provider.
 
-        @return: A L{Deferred<twisted.internet.defer.Deferred>} which fires with
-            L{None} when all bytes have been produced or with a
+        @return: A L{Deferred<twisted.internet.defer.Deferred>} which stops
+            production of data when L{Deferred.cancel} is called, and which
+            fires with L{None} when all bytes have been produced or with a
             L{Failure<twisted.python.failure.Failure>} if there is any problem
             before all bytes have been produced.
         """
@@ -469,7 +505,7 @@ class IRenderable(Interface):
         """
         Look up and return the render method associated with the given name.
 
-        @type name: C{str}
+        @type name: L{str}
         @param name: The value of a render directive encountered in the
             document returned by a call to L{IRenderable.render}.
 
@@ -502,7 +538,7 @@ class ITemplateLoader(Interface):
         """
         Load a template suitable for rendering.
 
-        @return: a C{list} of C{list}s, C{unicode} objects, C{Element}s and
+        @return: a L{list} of L{list}s, L{unicode} objects, C{Element}s and
             other L{IRenderable} providers.
         """
 
@@ -517,22 +553,22 @@ class IResponse(Interface):
 
     version = Attribute(
         "A three-tuple describing the protocol and protocol version "
-        "of the response.  The first element is of type C{str}, the second "
-        "and third are of type C{int}.  For example, C{('HTTP', 1, 1)}.")
+        "of the response.  The first element is of type L{str}, the second "
+        "and third are of type L{int}.  For example, C{(b'HTTP', 1, 1)}.")
 
 
-    code = Attribute("The HTTP status code of this response, as a C{int}.")
+    code = Attribute("The HTTP status code of this response, as a L{int}.")
 
 
     phrase = Attribute(
-        "The HTTP reason phrase of this response, as a C{str}.")
+        "The HTTP reason phrase of this response, as a L{str}.")
 
 
     headers = Attribute("The HTTP response L{Headers} of this response.")
 
 
     length = Attribute(
-        "The C{int} number of bytes expected to be in the body of this "
+        "The L{int} number of bytes expected to be in the body of this "
         "response or L{UNKNOWN_LENGTH} if the server did not indicate how "
         "many bytes to expect.  For I{HEAD} responses, this will be 0; if "
         "the response includes a I{Content-Length} header, it will be "
@@ -595,10 +631,10 @@ class _IRequestEncoder(Interface):
         Encode the data given and return the result.
 
         @param data: The content to encode.
-        @type data: C{str}
+        @type data: L{str}
 
         @return: The encoded data.
-        @rtype: C{str}
+        @rtype: L{str}
         """
 
 
@@ -608,7 +644,7 @@ class _IRequestEncoder(Interface):
 
         @return: If necessary, the pending data accumulated from previous
             C{encode} calls.
-        @rtype: C{str}
+        @rtype: L{str}
         """
 
 
